//<c>   Copyright 1990-1999 by Gerry J. Danen; all rights reserved.

//<t>   Microsoft C Version 3.0, 5.1 interface to the
//      Btrieve Record Manager, version 4, 5.0, 5.1


#include"gd_tools.h"
#include"gd_btrv.h"


//  For Compact, Large, and Huge memory models
//  LMODEL means 32-bit pointers in use

#if (defined(M_I86CM) || defined(M_I86LM) || defined(M_I86HM))
    #define LMODEL 1
#endif


// bt_procId (ProcId) is used for communicating with the Multi Tasking Version of
// Btrieve. It contains the process id returned from BMulti and should
// not be changed once it has been set.


static unsigned bt_procId = 0;          // initialize to no process id
static char bt_mu = 0;                  // flag set to true if MultiUser
static char bt_vCheck = 0;              // flag set to true if checked version


int btrv( int opCode, char posBlk[], char dataBuf[], int *dataLen,
          char keyBuf[], int keyNum )
{
    union REGS b_regs;                          /* define general registers */
    struct SREGS b_sregs;                       /* define segment registers */

    struct BTRIEVE_PARMS      /* structure passed to Btrieve Record Manager */
    {
        char *bufOffset ;                      /* callers data buffer offset */
#ifndef LMODEL
       int   bufSeg ;                        /* callers data buffer segment */
#endif
       int   bufLen ;                              /* length of data buffer */
       char *curOffset ;                      /* user position block offset */
#ifndef LMODEL
       int   curSeg ;                        /* user position block segment */
#endif
       char *fcbOffset ;                              /* offset of disk FCB */
#ifndef LMODEL
       int   fcbSeg ;                                /* segment of disk FCB */
#endif
       int   funCode ;                                /* requested function */
       char *keyOffset ;                     /* offset of user's key buffer */
#ifndef LMODEL
       int   keySeg ;                       /* segment of user's key buffer */
#endif
       char  keyLength ;                     /* length of user's key buffer */
       char  keyNumber ;                    /* key of reference for request */
       int  *statOffset ;                          /* offset of status word */
#ifndef LMODEL
       int   statSeg ;                            /* segment of status word */
#endif
       int   xFaceId ;                               /* language identifier */
    } btr;

    int stat = 0 ;                              /* status of Btrieve call */

    //  Check to see that the Btrieve Record Manager has been started.

    if ( ! bt_vCheck )                  /* if we don't know version of Btrieve yet */
    {
        bt_vCheck = 1 ;
//      b_regs.x.ax = 0x3000;                            /* check dos version */
//      int86x (0x21, &b_regs, &b_regs, &b_sregs);
//      if ( (b_regs.x.ax & 0x00FF) >= 3 )      /* if DOS version 3 or later */
        if ( _osmajor >= 3 )                    // if DOS version 3 or later
        {
            segread( &b_sregs );                //~~~ added 10-Dec-95
            b_regs.x.ax = _2FCODE ;
            int86x( BTR2_INT, &b_regs, &b_regs, &b_sregs );
            bt_mu = ((b_regs.x.ax & 0xFF) == 'M');/* if al is M, bmulti is loaded */
        }
    }

    if ( ! bt_mu )
    {                                           // if bmulti not loaded
        b_regs.x.ax = 0x3500 + BTR_INT;
        int86x (0x21, &b_regs, &b_regs, &b_sregs);
        if (b_regs.x.bx != BTR_OFFSET)
            return (BTR_ERR);
    }

//  Read segment registers and initialize segment part of addresses to
//  user's data segment.

    segread( &b_sregs );
#ifndef LMODEL
    btr.bufSeg = btr.curSeg = btr.fcbSeg = btr.keySeg = btr.statSeg = b_sregs.ss ;
#endif

//  Move user parameters to "btr", the block where Btrieve expects them.

    btr.funCode    = opCode ;
    btr.statOffset = &stat ;
    btr.fcbOffset  = posBlk ;
    btr.curOffset  = posBlk + 38 ;
    btr.bufOffset  = dataBuf ;
    btr.bufLen     = *dataLen ;
    btr.keyOffset  = keyBuf ;
    btr.keyLength  = (char) -1 ;                // use max since we don't know (-1 = 255!!!)
    btr.keyNumber  = keyNum ;
    btr.xFaceId    = VARIABLE_ID ;

//  Make call to the Btrieve Record Manager.

    b_regs.x.dx = (int) &btr;  /* parameter block is expected to be in DX */
    b_sregs.ds = b_sregs.ss;
    if ( ! bt_mu )
        int86x (BTR_INT, &b_regs, &b_regs, &b_sregs);
    else
    {                                                       /* call bmulti */
        while (1)
        {
            b_regs.x.ax = 1;                 /*  assume no proc id obtained yet */
            if ((b_regs.x.bx = bt_procId) != 0)            /* if we have a proc id */
                b_regs.x.ax = 2;                              /* tell bmulti that */
            b_regs.x.ax += _2FCODE;
            int86x (BTR2_INT, &b_regs, &b_regs, &b_sregs);
            if ((b_regs.x.ax & 0x00FF) == 0)    /* if call was processed */
                break ;                         /* by bmulti, leave loop */
        }
        if ( bt_procId == 0 )
            bt_procId = b_regs.x.bx ;
    }

    *dataLen = btr.bufLen ;
    return( stat );                             /* return status */
}


#if 0
void main()
{
    char x ;
    x= (char) -1;
    printf( "%c %d %X\n", x, x, x );
}
#endif
