//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Convert Date to File Date Stamp

//<y>   Y2K compliance depends on OS file stamp format

#include"gd_tools.h"
#include"gd_y2k.h"


UINT16  Date2Fstamp( INT16 year, INT16 month, INT16 day )
/*
  Syntax        stamp = Date2Fstamp( year, month, day );

                UINT16 stamp    16-bit date stamp
                INT16  year     Year (1980 - 2107)                      Y2K
                INT16  month    Month (1 - 12)
                INT16  day      Day of the month (1 - 31)

  Description   This function converts a date into the MS-DOS directory
                date stamp format.

                The date stamp is a date of the year packed into
                the following format:

                **********   bits   ***********
                1 1 1 1 1 1
                5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0

                y y y y y y y m m m m d d d d d = stamp

                where the year bits represent 0 - 127 (1980 to 2107).   Y2K

  Returns       stamp             Resulting 16-bit date stamp (0 if error).
*/
{
    UINT16      stamp = 0 ;

    if ( InRange( year, Y2K_DOS_FST_LOW, Y2K_DOS_FST_HI ) &&            // Y2K
         InRange( month, 1, 12 ) && InRange( day, 1, 31 ) )
    {
        stamp = ( ((year - Y2K_DOS_FST_LOW) << 9) | (month << 5) | day ); // Y2K
    }
    return( stamp );
}
