//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Find the Day of the Week for a Given Date

//<y>   Compliant as per dt_sliding_window_year()


#include"gd_tools.h"
#include"gd_y2k.h"

#pragma check_stack(off)


int     dt_dow( int dd, int mm, int yy )
/*
        Calculate the day of the week for any given date, where the returned
        day is an int from 1(Sunday) to 7(Saturday).  The function is based on
        Zeller's Congruence.  Reference:  Jack Purdum's C Programmer's Toolkit.

        Usage:  daynum = dt_dow( day, mth, year );

        Param:  int day,mth,year valid date for calculations
        Return: int daynum       day number 1 to 7
*/
{
    register int i ;

    yy = dt_sliding_window_year( yy );          // Y2K

    if ( mm > 2 )
    {
        mm -= 2 ;
    }
    else
    {
        mm += 10 ;
        yy -= 1 ;
    }
    i = ((13 * mm - 1) / 5) + dd + (yy % 100) + ((yy % 100) / 4) +
        ((yy / 100) / 4) - 2 * (yy / 100) + 77 ;
    i = i - 7 * (i / 7) ;
    return( i + 1 );
}


#if 0
static  int doit( int dd, int mm, int yy )
{
    static int n, n1 ;

    if ( dd==1 && mm==1 )
        printf( "..    %04d\n", yy );
//  n = dow ( dd, mm, yy );
//  if ( n<1 || n>7 )
//      printf( "%02d-%02d-%04d  dow() INDEX ERROR! - %d\n", dd, mm, yy, n );
    n1 = dt_dow ( dd, mm, yy );
    if ( n1<1 || n1>7 )
        printf( "%02d-%02d-%04d  dt_dow() INDEX ERROR! - %d\n", dd, mm, yy, n1 );
//  if ( n != n1 )
//      printf( "%02d-%02d-%04d  n=%d, n1=%d\n", dd, mm, yy, n, n1 );
    return( n1 );
}

void main( int argc, char *argv[] )
{
    int ch, dow, dd, mm, yy ;
    if ( argc != 4 )
    {
        fprintf( stderr, "\nUsage:\tDT_DOW dd mm yyyy\n\n\tWhere\n"
                         "\n\t\tdd is the starting day\n\t\tmm is the starting month\n\t\tyyyy is the starting year\n"
                         "\n\tSample:\n\n\t\tDT_DOW 26 12 1949\n" );
        exit(1);
    }
    dd = atoi( argv[1] );
    mm = atoi( argv[2] );
    yy = atoi( argv[3] );
    fprintf( stderr, "Hit any key to check the date, [Esc] to quit.\n" );
    while ( 1 )
    {
        dow = doit( dd, mm, yy );
        if ( _kbhit() )
        {
            ch = _getch();
            if ( ch == 27 )
            {
                printf( "%02d-%02d-%04d  <cancelled>\n", dd, mm, yy );
                exit( 1 );
            }
            fprintf( stderr, "%02d-%02d-%04d  %s\n", dd, mm, yy, DayName(dow) );
        }
        nextday( &dd, &mm, &yy );
        if ( dd==1 && mm==1 && yy==2000 )
            exit( 0 );
    }
}
#endif
#if 0
static  int dow_old( int dd, int mm, int yy )       // OLD routine!
{
    INT16 day_index ;
    INT32 y4, y100, y400, t1, t2, t3, t4 ;

    if ( mm < 3 )
    {
         mm += 12 ;
         yy -=  1 ;
    }
    y4   = yy /   4 ;
    y100 = yy / 100 ;
    y400 = yy / 400 ;
    t1   = (6 * (mm + 1)) / 10 ;
    t2   = dd + 2 * mm + t1 + yy + y4 - y100 + y400 + 2 ;
    t3   = t2 / 7 ;
    t3   = t3 * 1000 ;
    t4   = t2 * 1000 / 7 ;
    t4   = (t4 - t3) * 7 + 500 ;
    day_index = (INT16) (t4 / 1000) ;
    return( (day_index ? day_index : 7) );
}
#endif
#if 0
typedef struct
{
    unsigned first  :  1 ;
    unsigned filler : 15 ;
}   bitties ;


static  int dow_old2( int dd, int mm, int yy )      // another old routine
/*
        Calculate the day of the week for any given date, where the returned
        day is an int from 1(Sunday) to 7(Saturday).

        See Computer Language Dec/88, page 35, for an article explaining the
        "Zeller's congruence" that inspired this routine.  Note that the
        article contains several errors.

        Usage:  daynum = dt_dow( day, mth, year );

        Param:  int day,mth,year valid date for calculations
        Return: int daynum       day number 1 to 7
*/
{
    union
    {
        int     intflag ;
        bitties bitflag ;
    }   u ;
    int i, m, dow, century, yr_in_cent, adj1, adj2 ;

    if ( yy < 100 )
        yy += 1900 ;                            // Y2K
    if ( mm < 3 )
    {
        mm += 12 ;
        yy -=  1 ;
    }
    mm -= 2 ;
    m = 2 * mm ;
    u.intflag = 6868 ;
    for ( i=0 ; i<mm ; i++ )
    {
        u.intflag = u.intflag >> 1 ;
        if ( u.bitflag.first )
            m++ ;
    }
    century    = yy / 100 ;
    yr_in_cent = yy - century * 100 ;
    adj1 = century / 4 ;
    adj2 = yr_in_cent / 4 ;
    dow = ( m + (-2 * century+adj1 + yr_in_cent+adj2) + dd ) % 7 ;
    return( (dow<0) ? dow+8 : dow+1 );
}
#endif
