//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Format a Date Number into a String

//<y>   Y2K compliant


#include"gd_tools.h"
#include"gd_extrn.h"
#include"gd_y2k.h"


STRING  mth_name_(INT16);


INT16   FormatDate( INT32 indate, STRING pdest, INT16 outfmt )
/*
        Usage:  iret = FormatDate( indate, pdest, outfmt );

                INT16   iret    # of characters in destination string,
                                or ERROR (illegal data or format codes)
                INT32   indate  date to be converted (ymd)
                UCHAR   *pdest  pointer to output string
                INT16   outfmt  indicates how pdest is to be formatted:
                                0 = ddmmmyy
                                1 = dd-mmm-yy
                                2 = dd.mmm.yyyy
                                3 = dd month yyyy
                                4 = month dd, yyyy
                                5 = month
                                6 = month yyyy
                                7 = mmm-yyyy
                                8 = mmm
                                9 = dd-mmm-yyyy
                               10 = month dd
                               11 = mm/dd/yy
                               12 = yyyymmdd
*/
{
    UCHAR       day[4], month[11], year[6] ;
    INT16       yy, mm, dd, leng=4 ;

    *pdest = EOS ;
    if ( OUTRANGE(outfmt,0,9) )
        return( ERROR );                        // illegal format code
    if ( (outfmt > 2 && outfmt < 7) || outfmt == 10 )
        leng = 10 ;                             // full month name
    if ( DateSplit( indate, &yy, &mm, &dd ) == ERROR )
        return( ERROR );                        // illegal date
    if ( outfmt < 2 || outfmt == 11 )
    {
        yy = yy MOD 100;
        sprintf( year, "%02d", yy );            // 2 digit year
    }
    else
        sprintf( year, "%04d", yy );            // 4 digit year
    if ( outfmt == 4 )                          // month dd, yyyy
        sprintf( day, "%2d", dd );
    else
        sprintf( day, "%02d", dd );
    st_ltrim( day );
    if ( outfmt == 11 )                         // mm/dd/yy
    {
        sprintf( pdest, "%02d/%02d/%02d", mm, dd, yy );
        return( strlen(pdest) );
    }
    if ( outfmt == 12 )                         // yyyymmdd
    {
        sprintf( pdest, "%04d%02d%02d", yy, mm, dd );
        return( strlen(pdest) );
    }
    st_cpy( month, mth_name_(mm), leng );
    if ( outfmt == 4 )                          // month dd, yyyy
    {
        sprintf( pdest, "%s %s, %s", month, day, year );
        return( strlen(pdest) );
    }
    if ( outfmt == 10 )                         // month dd
    {
        sprintf( pdest, "%s %d", month, dd );
        return( strlen(pdest) );
    }
    if ( outfmt == 5 || outfmt == 8 )           // month only
    {
        st_cpy( pdest, month, 20 );
        return( strlen(pdest) );
    }

    if ( outfmt < 4 || outfmt == 9 )
        st_cpy( pdest, day, 20 );
    if ( outfmt == 1 || outfmt == 9 )           // dd-mmm-yy
        st_cat( pdest, "-", 20 );
    if ( outfmt == 2 )                          // dd.mmm.yyyy
        st_cat( pdest, ".", 20 );
    if ( outfmt == 3 )                          // dd month yyyy
        st_cat( pdest, " ", 20 );
    st_cat( pdest, month, 20 );
    if ( outfmt == 1 || outfmt == 7 || outfmt == 9 )
        st_cat( pdest, "-", 20 );               // dd-mmm-yy, mmm-yyyy, dd-mmm-yyyy
    if ( outfmt == 2 )                          // dd.mmm.yyyy
        st_cat( pdest, ".", 20 );
    if ( outfmt == 3 || outfmt == 6 )           // dd month yyyy or month yyyy
        st_cat( pdest, " ", 20 );
    st_cat( pdest, year, 20 );
    return( strlen(pdest) );
}


//      Return the name of the month.

STRING  mth_name_( INT16 n )
{
    static STRING name[] =
    {
        "???",          // Illegal Month
        "January", "February", "March", "April", "May", "June",
        "July", "August", "September", "October", "November", "December"
    };
    return( name[n] );
}


#if 0
void    main( void )
{
    UCHAR       reply[STR_LEN] ;
    INT32       ymd ;
    INT16       i, ret ;

    gd_init( 0 );
    kb_Break( BREAK_ON );                       // Allow Control-Break
    SetGlobalString( PGM_TITLE, "Testing the FormatDate routine" );
    NewScreen();
    FOREVER
    {
        EnterText( reply, 10, NO, 0, "Enter a date (yyyymmdd) [just <CR> quits]" );
        if ( ! strlen(reply) )
            gd_exit( 0, "" );
        ymd = atol( reply );
        NewScreen();
        DspMessage( 5,2, vid.normal_fg,vid.normal_bg, "ret   yyyymmdd  fmt  result" );
        LOOP( i, 0, 10, 1 )
        {
            ret = FormatDate( ymd, reply, i );
            DspMessage( 7+i,1, vid.normal_fg,vid.normal_bg, "%4d %10ld %4d  %s ", ret, ymd, i, reply );
        }
    }
}
#endif
