//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Convert File Date Stamp to Date

//<y>   Y2K compliance depends on OS file stamp


#include"gd_tools.h"
#include"gd_y2k.h"


INT16   Fstamp2date( UINT16 fdate, INT16 *pyear, INT16 *pmonth, INT16 *pday )
/*
  Syntax        ercode = Fstamp2date(fdate,pyear,pmonth,pday);

                int ercode      Error code is ERROR if date stamp is not a
                                legal date, otherwise OK.
                unsigned fdate  16-bit date stamp.
                int *pyear,*pmonth,*pday  Pointers to variables in which
                                the year, month, and day are returned.

  Description   This function converts a calendar date from the DOS directory
                date stamp format into separate year, month, and day.

                The date stamp (fdate) is a date of the year packed into the
                following format:

                      **********   bits   ***********
                      1 1 1 1 1 1
                      5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0

                      y y y y y y y m m m m d d d d d   = fdate

                where the year bits represent 0 - 127 (1980 to 2107).   Y2K

  Returns       ercode                  ERROR or OK
                *pyear,*pmonth,*pday    The date as numeric values
*/
{
    *pyear  = (fdate >> 9) + Y2K_DOS_FST_LOW ;  // Y2K
    *pmonth = (fdate >> 5) & 0x0f ;
    *pday   =  fdate       & 0x1f ;
    if ( (INRANGE(*pyear, Y2K_DOS_FST_LOW, Y2K_DOS_FST_HI)) &&
         (INRANGE(*pmonth, 1, 12)) && (INRANGE(*pday, 1, 31)) )  // Y2K
        return( OK );
    *pyear  = Y2K_DOS_FST_LOW ;                 // Y2K
    *pmonth = *pday = 1 ;
    return( ERROR );                            // Date is out of range
}
