//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Calculate New Date, using a Factor

//<y>   Y2K compliance depends on other functions


#include"gd_tools.h"
#include"gd_y2k.h"


INT32   NewDate( INT32 ymd, INT16 factor, INT16 addsub )
/*
        Given a date, calculate a new date, using a positive or negative
        increment.

        addsub is the Add/Subtract parameter, indicating which part of the
        date is to be in-/decremented:  3 = year    max 99,
                                        2 = month   max 99,
                                        1 = day     max 32000.
*/
{
    INT32       num1 ;
    INT16       valid, doing_y=FALSE, doing_m=FALSE, doing_d=FALSE ;
    INT16       y, m, d, days ;

    if ( ValidDate( ymd ) == ERROR )
        return( ERROR );                        // illegal date
    if ( factor == 0 )
        return( ymd );                          // same date if zero
    if ( OUTRANGE(addsub,1,3) )
        return( ERROR );                        // invalid add/sub indicator
    if ( addsub == 3 )
        doing_y = TRUE;
    if ( addsub == 2 )
        doing_m = TRUE;
    if ( addsub == 1 )
        doing_d = TRUE;

    //  Check for legal factor parameter

    if ( doing_y && (OUTRANGE( factor, -99, 99 )) )
        return( ERROR );
    if ( doing_m && (OUTRANGE( factor, -99, 99 )) )
        return( ERROR );
    if ( doing_d && (OUTRANGE( factor, -32000, 32000 )) )
        return( ERROR );

    DateSplit( ymd, &y, &m, &d );
    if ( doing_d )                              // DAY adjustment
    {
        if ( (num1 = Date2Num( ymd )) == ERROR )
            return( ERROR );
        else
            return( Num2Date(num1+factor) );
    }
    if ( doing_m )                              // MONTH adjustment
    {
        m = m + factor;
        valid = FALSE;
        do
        {
            if ( OUTRANGE( m, 1, 12 ) )
            {
                if ( m < 1 )
                {
                    m += 12;
                    y -= 1;
                }
                else
                {
                    m -= 12;
                    y += 1;
                }
            }
            else
            {
                valid = TRUE;
            }
        }   while ( !valid );
        ymd = (y*10000L) + (m*100L) + d;
        days = DaysInMonth( ymd );
        if ( d > days )                         // month-ending date error
            ymd = (y*10000L) + (m*100L) + days;
        return( ymd );
    }
    if ( doing_y )                              // YEAR adjustment
    {
        ymd = ((y+factor)*10000L) + (m*100L) + d;
        days = DaysInMonth( ymd );
        if ( d > days )                         // month-ending date error
            ymd = ((y+factor)*10000L) + (m*100L) + days;
        return( ymd );
    }
    return( ERROR );                            // catch-all error
}
