//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Return the System Date in a String

//<y>   Y2K compliance depends on system clock


#include"gd_tools.h"
#include"gd_y2k.h"


INT16   StrSDate( STRING pdate, INT16 *pyear, INT16 *pmonth, INT16 *pday )
/*
  Syntax        ercode = StrSDate(pdate,pyear,pmonth,pday);
                int ercode      Error code is -1 if current system date
                                is not a legal date.
                char *pdate     Pointer to string in which the date is
                                returned.  Must be at least [30] chars long.
                int *pyear,*pmonth,*pday  Pointers to variables in which
                                the year, month and day are returned.

  Description   This function returns the system date as integers and as a
                string containing the month and day of the week.  For example,
                19880928 is returned as:

                        Wednesday September 28, 1988

                Any illegal date returns ERROR and the NULL string.  The date
                string, pdate, must be large enough to accommodate the largest
                date string which can be 28 bytes long.

  Returns       int ercode      Returned error code
                char *pdate     Date string
                int *pyear,*pmonth,*pday  The date as numeric values.
*/
{
    UCHAR       work[21];
    INT32       yyyymmdd;
    INT16       yyyy, mm, dd;

    *pdate  = EOS;
    if ( dt_getsystemdate( &yyyymmdd, &yyyy, &mm, &dd ) == ERROR )
        return ( ERROR );
    if ( WhatDay( yyyymmdd, pdate, 1 ) == ERROR )
        return ( ERROR );
    if ( FormatDate( yyyymmdd, work, 4 ) == ERROR )
        return ( ERROR );
    st_cat( pdate, " ",  29 );
    st_cat( pdate, work, 29 );
    *pyear  = yyyy;
    *pmonth = mm;
    *pday   = dd;
    return( OK );
}
