//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Find the Day of the Week for a Given Date (yyyymmdd)

//<y>   Y2K compliance depends on other functions


#include"gd_tools.h"
#include"gd_y2k.h"


INT16   WhatDay( INT32 indate, UCHAR *pdest, INT16 outfmt )
/*
        Calculate the day of the week for any given date, where the day is
        an integer from 1(Sunday) to 7(Saturday).

        Usage:      day = WhatDay( indate, pdest, outfmt );

                    INT16   day     day number 1 to 7
                                    or ERROR (illegal data or format codes)
                    INT32   indate  date to be converted
                    UCHAR  *pdest   pointer to output string
                    INT16   outfmt  indicates how pdest is to be formatted:
                                    0 = 3 characters, e.g. Sun for Sunday
                                    1 = full day name, e.g. Sunday
*/
{
    INT16       yy, mm, dd, leng=4, day_index ;

    *pdest = EOS ;
    if ( ! InRange(outfmt, 0, 1) )
        return( ERROR );                        // illegal format code
    if ( outfmt == 1 )
        leng = 10 ;
    if ( DateSplit( indate, &yy, &mm, &dd ) == ERROR )
        return( ERROR );                        // illegal date
    day_index = dt_dow( dd, mm, yy );
    st_cpy( pdest, DayName(day_index), leng );
    return( day_index );
}


#if 0
static  void doit( int dd, int mm, int yy )
{   int n ;
    UCHAR str[21] ;
    n = WhatDay( ((INT32) yy * 10000L + (INT32) mm * 100L + (INT32) dd), str, 1 );
    if ( n<1 || n>7 )  strcpy( str, "*** ERROR ***" );
    fprintf( stderr, "%02d-%02d-%04d  %2d   %s\n", dd, mm, yy, n, str );
}
void main( int argc, char *argv[] )
{   int dd=26, mm=12, yy=1988, going=20 ;
    if ( argc == 4 )
    {   dd = atoi(argv[1]);  mm = atoi(argv[2]);  yy = atoi(argv[3]);
    }
    while (going)
    {   doit(dd,mm,yy);  nextday(&dd,&mm,&yy);  going-=1;
    }
}
#endif
