//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Copy a File

#include"gd_tools.h"
#include<types.h>
#include<io.h>
#include<fcntl.h>
#include<stat.h>


static  INT16   keep_destination(UCHAR *,UCHAR *);
static  INT16   get_yn(UCHAR *,UCHAR *,UCHAR *);


INT16   copy_file( UCHAR oldfile[], UCHAR newfile[], INT16 verify, INT16 confirm_overwrite, INT16 delete_after )
{
    UCHAR       *buf ;
    UINT32      f_date, f_time, outspace ;
    UINT16      bufsize, fds_av, fds_tot, fds_sec, bytesread=0, byteswritten=1 ;
    INT16       ercode, must_copy=TRUE, at_ro, at_hi, at_sy, at_vol, at_dir, at_ar ;
    int         inhandle=-1, outhandle=-1 ;
    FILE        *fptr ;

    //  Collect information about old file first!

    if ( st_eq( oldfile, newfile) )
    {
        printf( "Input file cannot be the same as output file: %s\n", oldfile );
        return( ERROR );                        // quit here!
    }

    if ( fptr = fopen( oldfile, "rb" ) )
    {
        GetFileInfo( fptr, &f_date, &f_time );
        fclose( fptr );
    }
    else
    {
        printf( "File %s not found\n", oldfile );
        return( ERROR );                        // quit here!
    }
    ercode = GetAttrib( oldfile, &at_ro, &at_hi, &at_sy, &at_vol, &at_dir, &at_ar );
    if ( ercode != 0 )
    {
        printf( "Could not get attributes from %s [error %d]\n", oldfile, ercode );
        return( ERROR );                        // quit here!
    }

    if ( confirm_overwrite )
    {
        if ( keep_destination( oldfile, newfile ) )
            goto reset_old ;
    }

    SetAttrib( oldfile, -1, -1, -1, -1 );       // turn attributes off
    SetAttrib( newfile, -1, -1, -1, -1 );

    //  Try rename only if "delete_after" is TRUE
    //  Then, if "rename" does not work, try copy...

    if ( delete_after )
        must_copy = FALSE ;
    if ( ! must_copy )
    {
        fl_delete( newfile );
        if ( rename( oldfile, newfile ) )
            must_copy = TRUE ;
    }
    if ( must_copy )
    {
        //  Allocate memory buffer

        bufsize = (_memavl() - BUFSIZ) / 16 ;
        bufsize *= 16 ;
        if ( bufsize < BUFSIZ )
            bufsize = BUFSIZ ;
        buf = (UCHAR *) malloc( bufsize );
        if ( ! buf )
        {
            printf( "Not enough memory to copy %s\n", oldfile );
            goto reset_old ;
        }
        if ( (inhandle = _open( oldfile, (int) (O_RDONLY | O_BINARY) ) ) == EOF )
        {
            printf( "Unable to open %s\n", oldfile );
            goto reset_old ;
        }
        else
        {
            fl_delete( newfile );
//~~~
            ercode = GetFreeDiskSpace( (newfile[0]-'A'), &fds_av, &fds_tot, &fds_sec );
            outspace = ( ((UINT32) fds_av) * ((UINT32) fds_sec) * ((UINT32) ercode) );
            if ( (UINT32) _filelength( inhandle ) > outspace )
            {
                printf( "Not enough disk space for %s\n", newfile );
                _close( inhandle );
                goto reset_old ;
            }
//~~~
        }
        if ( (outhandle = _open( newfile, (int) (O_RDWR|O_CREAT|O_BINARY), (int) (S_IREAD|S_IWRITE) ) ) == -1 )
        {
            printf( "Unable to open %s\n", newfile );
            _close( inhandle );
            goto reset_old ;
        }
        printf( "Copying %s to %s\n", oldfile, newfile );
        while ( byteswritten )
        {
            bytesread = read( inhandle, buf, bufsize );
            if ( bytesread == 0xFFFF )
                byteswritten = 0 ;
            else
                byteswritten = write( outhandle, buf, bytesread );
            if ( byteswritten != bytesread )
            {
                printf( "Error copying %s\n", oldfile );
                _close( inhandle  );
                _close( outhandle );
                goto reset_old ;
            }
        }
        _close( inhandle  );
        _close( outhandle );
        free( buf );
    }
    else
    {
        printf( "Moved %s to %s\n", oldfile, newfile );
        verify = FALSE ;                        // no need to verify
        delete_after = FALSE ;                  // already gone
    }

    if ( fptr = fopen( newfile, "rb" ) )
    {
        SetFileInfo( fptr, f_date, f_time );
        fclose( fptr );
    }
    ercode = SetAttrib( newfile, at_ro, at_hi, at_sy, at_ar );
    if ( delete_after )
    {
        if ( (fl_delete( oldfile )) != 0 )
            printf( "Could not delete file %s\n", oldfile );
    }
    if ( verify )
    {
        if ( compare_files( oldfile, newfile ) != OK )
        {
            printf( "Compare error\a\n" );
            return( ERROR );
        }
    }
    return( OK );

reset_old:
    ercode = SetAttrib( oldfile, at_ro, at_hi, at_sy, at_ar );
    if ( ercode != 0 )
        printf( "Could not reset attributes on %s [error %d]\n", oldfile, ercode );
    if ( buf )  free( buf );
    return( ERROR );
}


static  INT16   get_yn( UCHAR *m1, UCHAR *m2, UCHAR *m3 )
{
    INT16       ch ;

    printf( "%s%s%s", m1, m2, m3 );
    ch = _getch();
    ch = toupper( ch );
    if ( ch == 'Y' )
    {
        printf( "Yes\n" );
        return( TRUE );
    }
    printf( "No\n" );
    return( FALSE );
}


static  INT16 keep_destination( UCHAR *src, UCHAR *dest )
{
    struct stat instat, outstat ;
    INT16       inhandle, outhandle, keep_file=TRUE ;

    if ( (outhandle = _open( dest, (int) O_RDONLY )) > 0 )
    {
        if ( get_yn( "Destination file ", dest, " exists.  Overwrite it (y/N)? " ) )
        {
            inhandle = _open( src, (int) O_RDONLY );
            fstat( outhandle, &outstat );
            fstat( inhandle , &instat );
            _close( inhandle );
            if ( outstat.st_mtime > instat.st_mtime )
            {
                if ( get_yn( "OLDER file will replace NEWER file.  Overwrite (y/N)? ", "", "" ) )
                    keep_file = FALSE ;
            }
            else
                keep_file = FALSE ;
        }
    }
    else
        keep_file = FALSE ;
    if ( keep_file == FALSE )
    {
        if ( outhandle > 0 )
        {
            _close( outhandle );
            SetAttrib( dest, -1, -1, -1, -1 );  // turn attributes off
            fl_delete( dest );
        }
    }
    return( keep_file );
}


#if 0
void    main( int argc, char *argv[] )
{
    INT16       ret ;

    printf( "Copy File %s to %s\n", argv[1], argv[2] );
    ret = copy_file( argv[1], argv[2], YES, YES, NO );
    printf( "Return Code = \"%s\"\n", ret==OK ? "OK" : "ERROR" );
}
#endif
