//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   List a file on the screen

#include"gd_tools.h"
#include"gd_extrn.h"
#include"gd_kb.h"


#ifdef  DDOC
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

list_file:

        This function displays a text file on the screen.  The user is
        responsible for resetting to the proper screen width upon return from
        list_file.  The function uses a global buffer called
        "lf_work_" as well as a worker function, "lf_header".


        Syntax:

        BOOL list_file( STRING filename );

        STRING filename Name of file to be displayed.  No wildcards (e.g.
                        *.txt) are allowed.  File name may be passed as a
                        literal, or as a variable.

        Returns:
                        ERROR if file not found, otherwise OK.


        Example:

        error_code = list_file( "Student.Lis" );

- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#endif


FILE    *lf_fd ;
int     lf_abort ;
RC      lf_botcol ;

static  INT16   lf_header( STRING filename, STRING filedate, STRING filetime,
                           BOOL first )
{
    UCHAR       hdr[21] ;

    if ( first )
    {
        cls();
        ClearRect ( 1,1, 1,vid.cols, vid.sts_fg,vid.sts_bg );
        DspMessage( 1,2, vid.sts_fg,vid.sts_bg, filename );
        DspMessage( 1, (RC)(vid.cols-17), vid.sts_fg,vid.sts_bg, "%s %s",
                    filedate, filetime );
    }
    else
    {
        ClearRect( 2,1, (RC)(vid.rows-1), vid.cols, vid.normal_fg,vid.normal_bg );
    }
    ClearRect( vid.rows,1, vid.rows,vid.cols, vid.sts_fg,vid.sts_bg );
    sprintf( hdr, "%s %s", pgm_name, pgm_vers );
    DspMessage( vid.rows, (RC)(vid.cols-strlen(hdr)), vid.sts_fg,vid.sts_bg, "%s", hdr );
    DspMessage( vid.rows, 1, vid.sts_fg,vid.sts_bg, "Press [Enter] to continue, [Esc] to quit, X to cancel" );
    lf_botcol = sc_col();
    MoveCursor( 2, 1 );
    return( 2 );                                // new line number
}


static  BOOL    lf_key_quit( void )
{
    INT16       ch ;

    for ( ;; )
    {
        CursOn();
        ch = kb_getkey();
        CursOff();
        if ( ch == ESCAPE )                     // cancel
            return( TRUE );
        if ( ch == RETURN )                     // continue
            return( FALSE );
        if ( ch == 'X' || ch == 'x' )           // abort
        {
            lf_abort = TRUE ;
            return( TRUE );
        }
    }
}


BOOL    list_file( STRING fname )
{
    UINT32      ymd, hms ;
    char        real_file_name[257], *qfilename, file_time[8], file_date[12],
                line[134], *got ;
    UCHAR       lf_work_[256] ;                         // global for file display
    RC          ln ;

    lf_abort = FALSE ;
    cls();
    strcpy( real_file_name, fname );
    st_trim( real_file_name );
    _strupr( real_file_name );
    if ( ! strlen(real_file_name) )
        return( ERROR );
    if ( ! fl_exist( real_file_name ) )
    {
        ErrMsg( "File %s not found", real_file_name );
        return( ERROR );
    }
    lf_fd = fopen( real_file_name, "r" );
    if ( ! lf_fd )
        return( ERROR );
    qfilename = dr_q_fname( real_file_name );
    strcpy( real_file_name, qfilename );
    file_date[0] = file_time[0] = EOS ;
    GetFileInfo( lf_fd, &ymd, &hms );
    FormatDate( ymd, file_date, 1 );
    FormatTime( hms, file_time, 1, 0 );
    ln = lf_header( real_file_name, file_date, file_time, TRUE );
    do  {
        got = fgets( lf_work_, 255, lf_fd );
        if ( ! got )                            // EOF
            break ;
        st_rtrim( lf_work_ );                   // remove trailing \n
        StrDetab( line, lf_work_, vid.cols+1 );
        StrDelChar( line, FF );                 // remove formfeeds
        DspMessage( ln,1, vid.normal_fg,vid.normal_bg, "%s", line );
        ln += 1 ;
        if ( ln > (vid.rows-1) )
        {
            MoveCursor( vid.rows, lf_botcol );
            if ( lf_key_quit() )
            {
                ClearBottom();
                ln = 0 ;
                break ;
            }
            if ( lf_abort )
                goto done ;
            ln = lf_header( real_file_name, file_date, file_time, FALSE );
        }
    }   while ( got );
    if ( ln > 2 )
    {
        ClearRect( vid.rows,1, vid.rows,60, vid.normal_bg,vid.normal_fg );
        DspMessage( vid.rows, 1, vid.normal_bg,vid.normal_fg, " End of file   Press [Enter]" );
        sn_beep();
        lf_key_quit();
    }
    ClearBottom();
done:
    fclose( lf_fd );
    MoveCursor( vid.rows, 1 );
    return( OK );
}


void    list_files( STRING fspec )              // fspec can have wildcards
{
    int         done ;
    struct      find_t dta ;

    done = _dos_findfirst( fspec, REGULAR_FILES, &dta );
    if ( done )
    {
        ErrMsg( "%s not found", fspec );
        return ;
    }
    while ( ! done )
    {
        list_file( dta.name );
        if ( lf_abort )
            return ;
        done = _dos_findnext( &dta );
    }
}


#if 0
static  void vf_help( void )
{
    fprintf( stderr, "\nUsage:\n\tVF [fspec] [fspec] [fspec] ...\n\n" );
    fprintf( stderr, "\twhere\t[fspec] means a DOS file specification\n" );
    fprintf( stderr, "\tYou are allowed to enter multiple file specifications,\n\twith or without wildcards.\n\n" );
    fprintf( stderr, "\tExample:  VF *.bat *.txt\n" );
    gd_exit( 1, "" );
}


void    main( int argc, char *argv[] )
{
    extern int lf_abort ;

    gd_init( 0 );
    Allow_Break();                              // allow ^Break to abort
    SetGlobalString( PGM_NAME , "VF" );
    SetGlobalString( PGM_VERS , "V1.00" );
    if ( argc < 2 )
        vf_help();
    lf_abort = FALSE ;
    argc = 1 ;
    while ( strlen(argv[argc]) )
    {
        list_files( argv[argc++] );
        if ( lf_abort )
            gd_exit( 0, "" );
    }
    gd_exit( 0, "" );
}
#endif
