//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Format the Time for Printing

#include"gd_tools.h"


static  INT16 outfmt_leng_[] = { 8,6,9,12 };


INT16   FormatTime( INT32 tm_in, UCHAR *dest, INT16 iInFormat, INT16 iOutFormat )
/*
        Usage:      iret = FormatTime( tm_in, dest, iInFormat, iOutFormat );

                    INT16 iret  # of characters in destination string,
                                or ERROR (illegal data or format codes)
                    INT32 tm_in time to be converted
                    UCHAR *dest pointer to output string
                    INT16 iInFormat indicates contents of tm_in:
                                        0 = hhmmsstt
                                        1 = hhmmss
                                        2 = hhmm
                    INT16 iOutFormat indicates how dest is to be formatted:
                                        0 = hh:mmam/pm
                                        1 = hh:mm        (24-hour clock)
                                        2 = hh:mm:ss     (24-hour clock)
                                        3 = hh:mm:ss.tt  (24-hour clock)
*/
{
    UCHAR       cwork[6] ;
    INT16       hh, mm, ss, tt, am, leng ;

    *dest = EOS ;
    if ( OUTRANGE(iInFormat,0,2) || OUTRANGE(iOutFormat,0,3) )
        return( ERROR );                        // illegal format code
    leng = outfmt_leng_[ iOutFormat ];
    if ( iInFormat == 1 )
        tm_in = tm_in * 100L;
    if ( iInFormat == 2 )
        tm_in = tm_in * 10000L;
    tt     = (INT16) (tm_in MOD 100L);
    tm_in /= 100L;
    ss     = (INT16) (tm_in MOD 100L);
    tm_in /= 100L;
    mm     = (INT16) (tm_in MOD 100L);
    hh     = (INT16) (tm_in / 100L);
    if ( OUTRANGE(hh,0,23) || OUTRANGE(mm,0,59) || OUTRANGE(ss,0,59) || OUTRANGE(tt,0,99) )
        return(ERROR);                          // illegal time data
    if ( iOutFormat == 0 )                      // am/pm
    {
        if ( hh < 12 )
            am = TRUE;
        else
            am = FALSE;
        if ( hh > 12 )                          // change 13-23 to 1-11
            hh -= 12;
        if ( hh == 0 )                          // midnight
            hh = 12;
    }

#if 0
    sprintf( cwork, "%02d", hh );
    st_cat( dest, cwork, leng );
    st_cat( dest, ":",   leng );
    sprintf( cwork, "%02d", mm );
    st_cat( dest, cwork, leng );
#endif
    sprintf( dest, "%02d:%02d", hh, mm );

    if ( iOutFormat == 0 )                      // am/pm
    {
        if ( am )
            st_cat( dest, "am", leng );
        else
            st_cat( dest, "pm", leng );
    }
    if ( iOutFormat > 1 )                       // seconds
    {
        sprintf( cwork, ":%02d", ss );
        st_cat ( dest, cwork, leng );
    }
    if ( iOutFormat > 2 )                       // tics
    {
        sprintf( cwork, ".%02d", tt );
        st_cat ( dest, cwork, leng );
    }
    return( leng - 1 );
}
