//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Read and Write CMOS RAM


#include"gd_tools.h"

#pragma optimize("", off)


#define CMOS_CTRL_PORT  0x70    // CMOS RAM control port
#define CMOS_DATA_PORT  0x71    // CMOS RAM data port
#define CMOS_SREG_A     0x0a    // Status Register A
#define CMOS_SREG_D     0x0d    // Status Register D
#define CMOS_UPD_IN_PR  0x80    // Update in Progress


UINT    ut_get_cmos_byte(UINT cmos_address);


static void wait_for_UIP_to_clear( void )       // loop until "Update in Progress" bit is clear
{
    while ( ut_get_cmos_byte(CMOS_SREG_A) & CMOS_UPD_IN_PR )
        ;
}


UINT    ut_get_cmos_byte( UINT cmos_address ) // get a byte from CMOS RAM
{
    UINT cmos_value ;

    if ( cmos_address > 63 )                    // invalid
        return( 0 );

    _asm pushf                                  // save interrupt flag
    _disable();
    if ( cmos_address < CMOS_SREG_A )
        wait_for_UIP_to_clear();
    _outp( CMOS_CTRL_PORT, cmos_address | 0x80 );
    cmos_value = _inp( CMOS_DATA_PORT );
    _outp( CMOS_CTRL_PORT, CMOS_SREG_D );       // re-enable NMI
    _inp ( CMOS_DATA_PORT );                    // read CMOS_SREG_D
    _asm popf                                   // restore interrupt flag
    return( cmos_value );
}


void    ut_put_cmos_byte( UINT cmos_address, UINT cmos_value ) // put a value into a CMOS RAM byte
{
    if ( cmos_address > 63 )                    // invalid
        return;

    _asm pushf                                  // save interrupt flag
    _disable();
    if ( cmos_address < CMOS_SREG_A )
        wait_for_UIP_to_clear();
    _outp( CMOS_CTRL_PORT, cmos_address | 0x80 );
    _outp( CMOS_DATA_PORT, cmos_value );
    _outp( CMOS_CTRL_PORT, CMOS_SREG_D );       // re-enable NMI
    _inp ( CMOS_DATA_PORT );                    // read CMOS_SREG_D
    _asm popf                                   // restore interrupt flag
}


#if 0
void    main( void )
{
    UINT x, y, Loc, Data ;

    printf( "\n%36s\n\n  ", "CMOS RAM (128 bytes)" );
    for ( x = 0 ; x < 0x10 ; x++ )
        printf( " %2X", x );
    printf( "  0123456789ABCDEF\n  \n" );
    for ( x = 0 ; x < 8 ; x++ )
    {
        printf( "%X ", x );
        for ( y = 0 ; y < 0x10 ; y++ )
        {
            Loc = (x << 4) + y ;
            Data = ut_get_cmos_byte( Loc );
            printf( " %02X", Data );
        }
        printf( "  " );
        for ( y = 0 ; y < 0x10 ; y++ )
        {
            Loc = (x << 4) + y ;
            Data = ut_get_cmos_byte( Loc );
            if ( Data < 32 )
                printf( "." );
            else
                printf( "%c", (char) Data );
        }
        printf( "\n" );
    }
    printf( "\n\n" );
    x=0x19; printf( "Drive type 0 = %02Xh (@ %02Xh)\n",   ut_get_cmos_byte(x), x );
    x=0x1A; printf( "Drive type 1 = %02Xh (@ %02Xh)\n\n", ut_get_cmos_byte(x), x );

    x = cmos_get_base_memory();
    printf( "%5uK base memory     (@ 15h-16h)\n", x );
    x = cmos_get_extended_memory();
    printf( "%5uK extended memory (@ 17h-18h)\n", x );
    x = cmos_get_base_memory() + cmos_get_extended_memory();
    printf( "%5uK total memory installed according to CMOS\n", x );
    printf( "\n" );
}
#endif
