//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Get info about memory from CMOS
//      Ideas from Borland tips


#include"gd_tools.h"

/*
        The documented INT 15h, 88h may not always accurately return the amount
        of memory present in a PC.  This is a result of the method used by
        memory managers and other applications accessing/using extended memory.
        When extended memory is allocated, an application will usually hook the
        int 15h and monitor calls to function 88H; the new interrupt handler
        will only return the memory that is not used by the application.  Most
        Memory Managers will hook interrupt 15h and return a modified result
        when function 88h is called.  This makes this API unreliable.

        The correct and most direct method to get the amount of extended memory
        installed on a given machine, is to go through the CMOS.  The CMOS is
        organized into registers that can be queried by performing successive
        in and out instructions to ports 70H and 71H.  An out is done to port
        70H to tell the CMOS which register will be read from or written to.
        A read or write can then be done at port 71H to accomplish the desired
        read or write.
*/


static  UINT memory_according_to_cmos(UCHAR firstByte);


UINT    cmos_get_base_memory( void )            // amount of base memory (in K)
{
    return( memory_according_to_cmos( (UCHAR) 0x15 ) );
}


UINT    cmos_get_extended_memory( void )        // amount of extended memory (in K)
{
    return( memory_according_to_cmos( (UCHAR) 0x17 ) );
}


#pragma optimize("", off)


static  UINT memory_according_to_cmos( UCHAR firstByte ) // returns amount of memory (in K)
{
    UCHAR       secondByte=firstByte+1 ;
    UINT        Kmem ;

    _disable();
    _asm
    {
        ; Use the cmos data registers
        mov     al, firstByte   ; get least significant byte
        out     70h, al
        in      al, 71h
        mov     cl, al

        mov     al, secondByte  ; get most significant byte
        out     70h, al
        in      al, 71h
        mov     ch, al

        ; return extended memory in Kmem (AX!)
        mov     Kmem, cx
    }
    _enable();
    return( Kmem );
}


#if 0
void    main( void )
{
    UINT kBytes ;

    kBytes = cmos_get_base_memory();
    printf( "%5uK base memory\n", kBytes );
    kBytes = cmos_get_extended_memory();
    printf( "%5uK extended memory\n", kBytes );
    kBytes = cmos_get_base_memory() + cmos_get_extended_memory();
    printf( "%5uK total memory installed according to CMOS\n", kBytes );
}
#endif
