//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Get Equipment Info from BIOS


#include"gd_tools.h"
#include<bios.h>


typedef struct gd_bios_equip_info_struct_       // for this function only
{
    unsigned    disk_drives_installed      : 1 ;
    unsigned    math_coprocessor_installed : 1 ;
    unsigned    system_ram                 : 2 ;
    unsigned    initial_video_mode         : 2 ;
    unsigned    number_floppies            : 2 ;
    unsigned    dma_chip_installed         : 1 ;
    unsigned    serial_ports_installed     : 3 ;
    unsigned    game_adapter_installed     : 1 ;
    unsigned    internal_modem_installed   : 1 ;
    unsigned    printer_ports_installed    : 2 ;
}   bios_equipment_t ;


UINT    ut_get_equipment_info( equipment_t *e )
{
    union
    {
        unsigned int     bb ;
        bios_equipment_t bios ;
    }   b ;

    b.bb = _bios_equiplist();

#if 0
    printf( "disk_drives_installed      %d\n", b.bios.disk_drives_installed      );
    printf( "math_coprocessor_installed %d\n", b.bios.math_coprocessor_installed );
    printf( "system_ram                 %d\n", b.bios.system_ram                 );
    printf( "initial_video_mode         %d\n", b.bios.initial_video_mode         );
    printf( "number_floppies            %d\n", b.bios.number_floppies            );
    printf( "dma_chip_installed         %d\n", b.bios.dma_chip_installed         );
    printf( "serial_ports_installed     %d\n", b.bios.serial_ports_installed     );
    printf( "game_adapter_installed     %d\n", b.bios.game_adapter_installed     );
    printf( "internal_modem_installed   %d\n", b.bios.internal_modem_installed   );
    printf( "printer_ports_installed    %d\n", b.bios.printer_ports_installed    );
    printf( "\n" );
#endif

    e->math_coprocessor = b.bios.math_coprocessor_installed == 1 ;
    e->game_adapter     = b.bios.game_adapter_installed     == 1 ;
    e->internal_modem   = b.bios.internal_modem_installed   == 1 ;
    e->DMA_chip         = b.bios.dma_chip_installed         == 0 ;

    if ( b.bios.disk_drives_installed )
        e->diskette_drives = b.bios.number_floppies + 1 ;
    else
        e->diskette_drives = 0 ;

    if ( b.bios.initial_video_mode == 3 )       // both bits on
        e->screen_adapter = 0 ;                 // monochrome adapter
    else
        e->screen_adapter = 1 ;                 // graphics adapter

    e->ser_ports = b.bios.serial_ports_installed ;  // serial ports
    e->par_ports = b.bios.printer_ports_installed ; // parallel ports

    e->s_b_ram_banks = b.bios.system_ram + 1 ;  // banks of RAM on motherboard

    e->total_mem_size = _bios_memsize();        // total conventional RAM
    if ( e->total_mem_size == 639 )
        e->total_mem_size = 640 ;               // adjust for certain machines

    return( b.bb );
}


#if 0
void    main( void )
{
    equipment_t eq ;

    ut_get_equipment_info( &eq );

    printf( "Screen adapter ..................... %s\n" , eq.screen_adapter ? "Graphics" : "Monochrome" );
    printf( "Diskette drives .................... %u\n" , eq.diskette_drives );
    printf( "Banks of memory on system board .... %u\n" , eq.s_b_ram_banks );
    printf( "Total available conventional RAM ... %uK\n", eq.total_mem_size );
    printf( "Serial ports ....................... %u\n" , eq.ser_ports );
    printf( "Printer ports ...................... %u\n" , eq.par_ports );
    printf( "Math coprocessor ................... %s\n" , eq.math_coprocessor ? "Yes" : "No" );
    printf( "Game adapter ....................... %s\n" , eq.game_adapter     ? "Yes" : "No" );
    printf( "Internal modem ..................... %s\n" , eq.internal_modem   ? "Yes" : "No" );
    printf( "DMA chip ........................... %s\n" , eq.DMA_chip         ? "Yes" : "No" );
}
#endif
