//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Check for wild NULL pointers

/*
        So you've got a null pointer...

        To get specific information about just where the error is occurring,
        use the function below.  Place the first call to null_error() at the
        very start of the program, in order to initialize it, and subsequent
        calls scattered strategically through your program.

        For tidiness sake I make all calls to null_error() part of assert()
        statements:

                assert(!null_error());

        That way defining #NDEBUG automatically removes all such calls and I
        can leave them scattered around in the meantime, just in case.

        Gives a warning when compiling in the LARGE model, so it may or may
        not work there...
*/
#define NDEBUG

#include"gd_tools.h"
#include<assert.h>


int     null_error( void )
/*
        Use once at start of program to initialize.  Later calls return 0 if
        NULL pointer region unchanged, 1 if NULL pointer region of memory has
        been changed.
*/
{
    static UCHAR record[16], first_time = 0 ;
    int i ;

    if ( first_time == 0 )
    {
        first_time = 1 ;
        for ( i = 0 ; i < 16 ; i++ )
            record[i] = (UCHAR) (*((UCHAR *)i) );
#ifndef NDEBUG
        printf( "null_error(): " );
        for ( i = 0 ; i < 16 ; i++ )
        {
            if ( isprint(record[i]) )
                printf( "%c", record[i] );
            else
                printf( "." );
        }
        printf( "\n" );
#endif
    }
    else
    {
#ifndef NDEBUG
        printf( "null_error():" );
        for ( i = 0 ; i < 16 ; i++ )
        {
            printf( " %02X", (int) record[i] );
        }
        printf( "\n             " );
        for ( i = 0 ; i < 16 ; i++ )
        {
            printf( " %02X", (int) (*((UCHAR *)i)) );
        }
        printf( "\n" );
#endif
        for ( i = 0 ; i < 16 ; i++ )
        {
            if ( ((int) *((UCHAR *)i)) != ((int) record[i]) )
                return( 1 );
        }
    }
    return( 0 );
}


#if 0
void    main( void )
{
    char *p ;

    null_error();
#ifdef NDEBUG
    printf( "No NULL checking\n" );
#else
    printf( "Checking for wild NULL pointers...\n" );
#endif
    printf( "Wild pointer: %s\n", p );
    assert(!null_error());
    strcpy( p, "something" );
    assert(!null_error());
    p = 0 ;
    strcpy( p, "something" );
    assert(!null_error());
    exit( 0 );
}
#endif
