//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Create a "SOUNDEX" code


#include"gd_tools.h"


#define         MAX_NUMBER      5


INT16   Soundex( STRING soundex, STRING in_string )
/*
        Create a soundex code in "soundex".

        The calling routine must have allocated enough space in "soundex" for
        "MAX_NUMBER" characters plus the terminating NULL.  Note that some
        algorithms call for less characters!

        Algorithm:
                1. Change all lowercase to uppercase
                2. Retain first letter of input string (must be alpha)
                3. Ignore the letters A, E, H, I, O, U, W, Y, and any other
                   non-alphabetic characters
                4. Substitute the following:
                        1 = B F P V
                        2 = C G J K Q S X Z
                        3 = D T
                        4 = L
                        5 = M N
                        6 = R
                5. Ignore identical letters next to each other
                6. Add trailing zeros if the length is less than "MAX_NUMBER"
                   characters

*/
{
//  ----------------->  Conversion table:  abcdefghijklmnopqrstuvwxyz
    static  UCHAR       soundexcode[27] = "01230120022455012623010202" ;
    UCHAR               ch, prev_ch ;
    INT16               i ;

    st_cpy( soundex, "000000", MAX_NUMBER+1 );  //  add trailing zeros
    if ( (!strlen(in_string)) || (!isalpha(*in_string)) )
        return( ERROR );                //  NULL string or 1st char not alpha
    prev_ch     = EOS ;
    *soundex++  = UP_UCHAR(*in_string); //  keep 1st alpha character
    *in_string++ ;
    for ( i=2 ; (i NOT_GR MAX_NUMBER) && *in_string ; *in_string++ )
    {
        ch = UP_UCHAR(*in_string) ;
        if ( (isalpha(ch)) && (soundexcode[ch-'A'] != ZERO) && (ch != prev_ch) )
        {
            *soundex++ = soundexcode[ch-'A'] ;
            i += 1 ;
        }
        prev_ch = ch ;
    }
    return( OK );
}


#if 0
void main()
{
    UCHAR sdex[81], name[81] ;

    printf( "\n\nSOUNDEX V1.10\t\tGenerate Soundex Codes\n\t\t\t(c) 1988-1996 by Gerry J Danen\n" );
loop:
    printf( "\nName (just [Enter] to quit): " );
    gets( name );
    if ( ! strlen(name) )
        exit( 0 );
    if ( Soundex( sdex, name ) )
        printf( "ERROR: name must start with alpha character, not \"%c\".\a\n", name[0] );
    else
        printf( "Soundex Code for name \"%s\" = %s \n", name, sdex );
    goto loop ;
}
#endif
